//
//  starter_pdfimpose.m
//  PDFlib
//

#import "starter_pdfimpose.h"
#import "PDFlib_ios/PDFlib_objc.h"
#import <UIKit/UIKit.h>

@implementation starter_pdfimpose

+ (NSData *) createPdf:(NSError**)error {
    NSData *pdfdata=nil;
    
    /* This is where font/image/PDF input files live. Adjust as necessary. */
    NSString *searchpath = [[NSBundle mainBundle] resourcePath];
    NSString *title = @"PDF Impose";
    NSString *infile1 = [[NSBundle mainBundle] pathForResource:@"PDFlib-datasheet" ofType:@"pdf"];
    NSString *infile2 = [[NSBundle mainBundle] pathForResource:@"PLOP-datasheet" ofType:@"pdf"];
    NSString *infile3 = [[NSBundle mainBundle] pathForResource:@"pCOS-datasheet" ofType:@"pdf"];
    NSArray *pdffiles = @[infile1, infile2, infile3];

    NSInteger i, col = 0, row = 0;
    NSString           *optlist;
    double scale = 1;          // scaling factor of a page
    double rowheight;          // row height for the page to be placed
    double colwidth;           // column width for the page to be placed
    
    double sheetwidth = 595;   // width of the sheet
    double sheetheight = 842;  // height of the sheet
    int maxcol=3, maxrow=4;    // maxcol x maxrow pages will be placed on one sheet
    
    @try {
        /* create a new PDFlib object */
        PDFlib *pdflib = [[PDFlib alloc] init];
        
        [pdflib set_option:@"errorpolicy=exception"];

        // Set the search path for fonts and PDF files
        [pdflib set_option:[NSString stringWithFormat:@"SearchPath={{%@}}", searchpath]];
        [pdflib begin_document:@"" optlist:@""];
        
        [pdflib set_info:@"Creator" value:@"PDFlib Cookbook"];
        [pdflib set_info:@"Title" value:title];

        /* ---------------------------------------------------------------------
         * Define the sheet width and height,  the number of rows and columns
         * and calculate the scaling factor and cell dimensions for the
         * multi-page imposition.
         * ---------------------------------------------------------------------
         */
        if (maxrow > maxcol)
            scale = 1.0 / maxrow;
        else
            scale = 1.0 / maxcol;
        
        rowheight = sheetheight * scale;
        colwidth = sheetwidth * scale;
        
        BOOL pageopen = false; // is a page open that must be closed?

        /* Loop over all input documents */
        for (i=0; i < [pdffiles count];)
        {
            NSInteger indoc, maxpage, pageno, page;
            
            /* Open the input PDF */
            indoc = [pdflib open_pdi_document:pdffiles[i] optlist:@""];
            maxpage = (NSInteger) [pdflib pcos_get_number:indoc path:@"length:pages"];
            
            /* Loop over all pages of the input document */
            for (pageno = 1; pageno <= maxpage; pageno++)
            {
                page = [pdflib open_pdi_page:indoc pagenumber:pageno optlist:@""];
                
                /* Start a new page */
                if (!pageopen) {
                    [pdflib begin_page_ext:sheetwidth height:sheetheight optlist:@""];
                    pageopen = true;
                }
                
                /* The save/restore pair is required to get an independent
                 * clipping area for each mini page. Note that clipping
                 * is not required for the imported pages, but affects
                 * the rectangle around each page. Without clipping we
                 * would have to move the rectangle a bit inside the
                 * imported page to avoid drawing outside its area.
                 */
                [pdflib save];
                
                // Clipping path for the rectangle
                [pdflib rect:(col * colwidth) y:(sheetheight - (row + 1) * rowheight) width:colwidth height:rowheight];

                [pdflib clip];
                
                optlist = [NSString stringWithFormat:@"boxsize={%f %f} fitmethod=meet", colwidth, rowheight];
                
                // Fit the imported page in the box
                [pdflib fit_pdi_page:page x:(col * colwidth) y:(sheetheight - (row + 1) * rowheight) optlist:optlist];
                
                [pdflib close_pdi_page:page];
                
                /* Draw a frame around the mini page */
                [pdflib set_graphics_option:[NSString stringWithFormat:@"linewidth=%f", scale]];
                [pdflib rect:col * colwidth y:sheetheight - (row + 1) * rowheight width:                       colwidth height:rowheight];
                [pdflib stroke];
                
                [pdflib restore];
                
                col++;
                
                // Start a new row if the current row is full
                if (col == maxcol) {
                    col = 0;
                    row++;
                }
                
                // Close the page if it is full
                if (row == maxrow) {
                    row = 0;
                    [pdflib end_page_ext:@""];
                    pageopen = false;
                }
            }
            [pdflib close_pdi_document:indoc];
            i++;
        }
        
        if (pageopen) {
            [pdflib end_page_ext:@""];
        }
        
        [pdflib end_document:@""];

        pdfdata = [pdflib get_buffer];
    } 
    @catch (PDFlibException *ex) { 
        *error = [[NSError alloc] initWithDomain:[NSString stringWithFormat:@"PDFlib error %ld in '%@': %@", (long)[ex get_errnum], [ex get_apiname], [ex get_errmsg]] code:[ex get_errnum] userInfo:nil];
    } 
    return pdfdata;
}

@end
